/*
    4-26-05
    Copyright Spark Fun Electronics 2005
    Nathan Seidle 
    spark@sparkfun.com
    
    Basic control and monitoring of a Toaster oven for SMD reflowing.

    The unit waits for two commands:
    
    0x56 - Relay On : Activates the heating elements
    0x43 - Relay Off : Deactivates the heating elements
    
    0x78 - Temperature Check : Unit takes a temperature reading and responds with the 10-bit reading in two bytes
    
    Safety settings : The reflow toaster controller will shut off the relay if it does not 
    receive the Relay On command after 30 seconds.
    
    We use a simple, cheap, thermocouple with the expensive AD595AQ Amplifier
    
*/
#define Clock_8MHz
#define Baud_9600

#define STATUS  PORTB.3
#define RELAY   PORTB.4

#define OFF 0
#define ON  1

#include "d:\Pics\c\16F88.h" // Device hardware map
#include "d:\Pics\c\int16CXX.H" // Device interrupt definitions

//#pragma config |= 0x2902
#pragma origin 4

/*
//Interrupt Vectors
//============================
interrupt serverX( void)
{
    int_save_registers
    char sv_FSR = FSR;  // save FSR if required

    /*
    if(INTF) //RB0 Change Interrupt
    {
        uns8 j, i;
        
        //Pause for 8.68us during start bit.
        //Then pause for ~4.34us to read middle of first data bit
        //Total 13.02us
        for(i = 0 ; i < 7 ; i++);
        
        for(j = 0 ; j < 8 ; j++)
        {
            data_byte_in = rr(data_byte_in);
            data_byte_in.7 = !Serial_In_BB; //Sample the port, should be in middle of the bit ~8.68us
            for(i = 0 ; i < 3 ; i++); nop2();//9us
        }

        //Clear INT Flag
        INTF = 0;
    }
    */

    FSR = sv_FSR;               // restore FSR if saved
    int_restore_registers 
}
//============================
*/

//Function definitions
//============================
#include "d:\Pics\code\Delay.c"   // Delays
#include "d:\Pics\code\Stdio.c"   // Print routines

void boot_up(void);
uns16 check_temp(void);
//============================
//End Function Definitions

//Global Variables
//============================

//============================
//End Global Variables

void main()
{
    uns8 choice;
    uns16 adc_reading;
    uns24 temperature;

    boot_up();

    while(1)
    {
        //Check to see if we need to turn on/off the elements
        //====================================================
        choice = scanc();
        if(choice == '1') RELAY = ON;
        if(choice == '2') RELAY = OFF;
        //====================================================
        
        //Get temp reading
        //====================================================
        adc_reading = check_temp();
        printf("ADC = %d ", adc_reading);

        //Now take this 10 bit number and convert it to C
        //1024bits / 5V = adc_reading / ?V
        //1024bits / 5000mV = adc_reading / ?mV
        temperature = adc_reading * (uns24)5000; //75 * 5000 = 375000
        temperature /= 1024; //375000 / 1024 = 366mV
        temperature /= 10; //366mV / 10mV/C = 36.6C = 97.88F
        
        printf("It's %d C\r", temperature);
        //====================================================

        delay_ms(50);
        STATUS ^= 1;
    }
    
}//End Main

//Init vars and ports
void boot_up(void)
{
    
    OSCCON = 0b.0111.0000; //Setup internal OSC to 8MHz
    
    ANSEL = 0b.0000.0001; //Turn RA0/AN0 to Analog
    
    PORTA = 0b.0000.0000;
    TRISA = 0b.0000.0001;  //0 = Output, 1 = Input (Temp on RA0)

    PORTB = 0b.0000.0000;
    TRISB = 0b.0000.0100;  //0 = Output, 1 = Input (RX on RB2)
    
    //Setup the hardware UART module
    //=============================================================
    SPBRG = 51; //8MHz for 9600 inital communication baud rate
    TXSTA = 0b.0010.0100; //8-bit asych mode, high speed uart enabled
    RCSTA = 0b.1001.0000; //Serial port enable, 8-bit asych continous receive mode
    //=============================================================

    //Make sure the oven is off at startup
    RELAY = OFF;
    /*
    //Setup Timer1 for delay between measurements
    T1CON = 0b.0011.0000; //Prescale of 1:8 - 1 timer click = 1.6us

    //Enable Interrupts
    INTEDG = 1; //Int on rising edge change
    INTE = 1;   //Enable the RB0 Int, for incoming commands
    GIE = 1;
    */
}    

//Returns the current temperature
#define AVG_AMT 16
uns16 check_temp(void)
{
    uns8 x;
    uns16 amount, total = 0;

    //Read Channel 1 - RA1 for X_ANALOG - using Vrefs+/- as inputs
    ADCON0 = 0b.0100.0001; //Select Fosc/16 for 8mhz, and channel RA0/AN0, A/D on
    ADCON1 = 0b.1100.0000; //Right justified, ADCS2 = 1 

    for(x = 0 ; x < AVG_AMT ; x++)
    {
        delay_ms(1);
        
        GO = 1; //Convert RA1 to digital
    
        while(GO == 1);
    
        amount.high8 = ADRESH;
        amount.low8 = ADRESL;
        
        total += amount; //Add on to the total
    }
    amount = total / AVG_AMT;
    
    return(amount);
}

//Reads e_data from the onboard eeprom at e_address
uns8 onboard_eeread(uns8 e_address)
{
    EEPGD = 0; //Point to EEPROM Memory

    //Do a read
    EEADR = e_address; //Set the address to read
    RD = 1; //Read it
    
    return(EEDATA); //Read that EEPROM value
}    

//Write e_data to the onboard eeprom at e_address
void onboard_eewrite(uns8 e_data, uns8 e_address)
{
    bit temp_GIE = GIE;
    
    EEPGD = 0; //Point to EEPROM data block
    FREE = 0; //Preform write only  

    EEIF = 0; //Clear the write completion intr flag
    EEADR = e_address; //Set the address
    EEDATA = e_data; //Give it the data
    WREN = 1; //Enable EE Writes
    GIE = 0; //Disable Intrs
    
    //Specific EEPROM write steps
    EECON2 = 0x55;
    EECON2 = 0xAA;
    WR = 1;
    //Specific EEPROM write steps

    while(EEIF == 0); //Wait for write to complete
    EEIF = 0; //Clear the write completion intr flag

    WREN = 0; //Disable EEPROM Writes

    GIE = temp_GIE; //Set GIE to its original state
}

